unit hiFileToolsEx;

interface

uses Kol,Share,windows;

type
  THIFileToolsEx = class
    private
      FThread:PThread;
      File1,File2:string;
      Flags:dword;
      FTotalSize, FByteTransfered, FCounter:integer;
      pbCancel:dword;
      function GetErrorCode:integer;
      function ThreadCopyProc(Sender:PThread):Integer;
      function ThreadMoveProc(Sender:PThread):Integer;
      procedure GenerateSuccess;
      procedure GenerateError;
    public
      _prop_FileName:string;
      _prop_NewFileName:string;
      _prop_DelToRecycle:byte;
      _prop_Replace:byte;
      _prop_InNewThread:byte;
      _prop_Precision:byte; //
      _data_NewFileName:THI_Event;
      _data_FileName:THI_Event;
      _data_Replace:THI_Event;
      _event_onEnd:THI_Event;
      _event_onError:THI_Event;
      _event_onStart:THI_Event;
      _event_onProgress:THI_Event;
    
      procedure _work_doMove(var _Data:TData; Index:word);
      procedure _work_doCopy(var _Data:TData; Index:word);
      procedure _work_doDelete(var _Data:TData; Index:word);
      procedure _work_doFileExists(var _Data:TData; Index:word);
      procedure _work_doMoveProgress(var _Data:TData; Index:word);
      procedure _work_doCopyProgress(var _Data:TData; Index:word);
      procedure _work_doCancel(var _Data:TData; Index:word);
    
      procedure _var_TotalSize(var _Data:TData; Index:word);
      procedure _var_TransferedSize(var _Data:TData; Index:word);
         
    end;
    
  {$ifdef F_P}
  function MoveFileWithProgress(lpExistingFileName, lpNewFileName: PChar; lpProgressRoutine: pointer; lpData: Pointer; dwFlags: DWORD): BOOL; stdcall; external 'kernel32.dll' name 'MoveFileWithProgressA';
  function CopyFileEx(lpExistingFileName, lpNewFileName: PChar; lpProgressRoutine: pointer; lpData: Pointer; pbCancel: PBool; dwCopyFlags: DWORD): BOOL; stdcall; external 'kernel32.dll' name 'CopyFileExA';
  {$endif}
  
implementation

uses hiStr_Enum;

procedure THIFileToolsEx._work_doMove;
var
  flag:dword;
begin
  File1 := ReadString(_Data,_data_FileName, _prop_FileName);
  File2 := ReadString(_Data,_data_NewFileName, _prop_NewFileName);
  if ReadInteger(_Data,_data_Replace, _prop_Replace) = 0 then flag:=0 else flag:=1;
  if MoveFileEx(PChar(File1),PChar(File2), flag + MOVEFILE_COPY_ALLOWED) then _hi_CreateEvent(_Data,@_event_onEnd) else _hi_CreateEvent(_Data,@_event_onError, GetErrorCode);
end;

procedure THIFileToolsEx._work_doCopy;
var
  flag:bool;
begin
  File1 := ReadString(_Data,_data_FileName, _prop_FileName);
  File2 := ReadString(_Data,_data_NewFileName, _prop_NewFileName);
  if ReadInteger(_Data,_data_Replace, _prop_Replace) = 0 then flag:=true else flag:=false;
  if CopyFile(PChar(File1),PChar(File2), flag) then _hi_CreateEvent(_Data,@_event_onEnd) else _hi_CreateEvent(_Data,@_event_onError, GetErrorCode);
end;

procedure THIFileToolsEx._work_doDelete;
var F1:string;
  function DeleteFiles( const DirPath: String ): Boolean;
  var
    Files, Name: String;
  begin
    Files := GetFileListStr( ExtractFilePath( DirPath ), ExtractFileName( DirPath ) );
    Result := TRUE;
    while Files <> '' do
    begin
      Name := fParse( Files, ';' );
      Result := Result and DeleteFile( PChar( Name ) );
    end;
  end;
begin
   F1 := ReadString(_Data,_data_FileName,_prop_FileName);
   if _prop_DelToRecycle = 0 then
      DeleteFile2Recycle(f1)
   else DeleteFiles(F1);
   if not FileExists(F1) then
     _hi_CreateEvent(_Data,@_event_onEnd);
end;

procedure THIFileToolsEx._work_doFileExists;
begin
  _hi_CreateEvent(_Data,@_event_onEnd,integer(FileExists(ReadString(_Data,_data_FileName,''))));
end;

function THIFileToolsEx.GetErrorCode;
var lsterr:dword;
begin
  lsterr:=GetLastError;
  case lsterr of
    2: result:=0; //    
    32: result:=1; //       
    5: result:=2;  //     " " / ""
    80: result:=3; //    ,  Replace=false
    1235: result:=4; //  
    3: result:=5; //    
  else 
    result:=lsterr;
  end;
end;


// - - -    - - - //


function CopyProgressCallback(TotalFileSize,TotalBytesTransferred,StreamSize,StreamBytesTransferred:LARGE_INTEGER; dwStreamNumber, dwCallbackReason:dword; hSourceFile, hDestinationFile:HWND; lpData:pointer):dword; stdcall;
begin                                                                                                                         
  if THIFileToolsEx(lpData).pbCancel = 0 then result:=PROGRESS_CONTINUE else result:=PROGRESS_CANCEL;
  
  case dwCallbackReason of
    CALLBACK_CHUNK_FINISHED:
    begin
      THIFileToolsEx(lpData).FByteTransfered:=TotalBytesTransferred.LowPart;
      if THIFileToolsEx(lpData).FCounter = 0 then   //
      begin    
        _hi_OnEvent(THIFileToolsEx(lpData)._event_onProgress, integer(TotalBytesTransferred.LowPart));
        THIFileToolsEx(lpData).FCounter := THIFileToolsEx(lpData)._prop_Precision;
      end
      else THIFileToolsEx(lpData).FCounter := THIFileToolsEx(lpData).FCounter - 1;  //
      
            
    end;
    CALLBACK_STREAM_SWITCH:
    begin
      THIFileToolsEx(lpData).FCounter := THIFileToolsEx(lpData)._prop_Precision; //
      THIFileToolsEx(lpData).FTotalSize:=TotalFileSize.LowPart;
      _hi_OnEvent(THIFileToolsEx(lpData)._event_onStart, integer(TotalFileSize.LowPart)); 
    end
  end;
  
end;


procedure THIFileToolsEx._work_doMoveProgress;
begin
  pbCancel:=0;
  FTotalSize:=0;
  FByteTransfered:=0;

  File1 := ReadString(_Data,_data_FileName, _prop_FileName);
  File2 := ReadString(_Data,_data_NewFileName, _prop_NewFileName);
  if ReadInteger(_Data,_data_Replace, _prop_Replace) = 0 then Flags:=0 else Flags:=1;
  
  if _prop_InNewThread = 0 then
    if MoveFileWithProgress(PChar(File1),PChar(File2), @CopyProgressCallback, self, Flags + MOVEFILE_COPY_ALLOWED) then GenerateSuccess else GenerateError
  else
  begin
    FThread:={$ifdef F_P}NewThreadforFPC{$else}NewThread{$endif};
    FThread.AutoFree:=true;  
    FThread.OnExecute:=ThreadMoveProc;
    FThread.Resume;
  end;   
end;

procedure THIFileToolsEx._work_doCopyProgress;
begin
  pbCancel:=0;
  FTotalSize:=0;
  FByteTransfered:=0;
  
  File1 := ReadString(_Data,_data_FileName, _prop_FileName);
  File2 := ReadString(_Data,_data_NewFileName, _prop_NewFileName);  
  if ReadInteger(_Data,_data_Replace, _prop_Replace) = 0 then Flags:=1 else Flags:=0;

  if _prop_InNewThread = 0 then
    if CopyFileEx(PChar(File1),PChar(File2),@CopyProgressCallback, self, pbool(@pbCancel), Flags) then GenerateSuccess else GenerateError
  else
  begin
    FThread:={$ifdef F_P}NewThreadforFPC{$else}NewThread{$endif};
    FThread.AutoFree:=true;  
    FThread.OnExecute:=ThreadCopyProc;
    FThread.Resume;
  end;   
end;

procedure THIFileToolsEx._work_doCancel;
begin
  pbCancel:=1;
end;

function THIFileToolsEx.ThreadCopyProc(Sender:PThread):Integer;
begin
  result:=0; //  ,       8))  ( )
  if CopyFileEx(PChar(File1),PChar(File2),@CopyProgressCallback, self, pbool(@pbCancel), Flags) then FThread.Synchronize(GenerateSuccess) else GenerateError;
end;

function THIFileToolsEx.ThreadMoveProc(Sender:PThread):Integer;
begin
  result:=0; // ( )
  if MoveFileWithProgress(PChar(File1),PChar(File2), @CopyProgressCallback, self, Flags + MOVEFILE_COPY_ALLOWED) then FThread.Synchronize(GenerateSuccess) else GenerateError;
end;

procedure THIFileToolsEx.GenerateError;
begin
  _hi_OnEvent(_event_onError, GetErrorCode);
end;

procedure THIFileToolsEx.GenerateSuccess;
begin
  _hi_OnEvent(_event_onEnd);
end;

procedure THIFileToolsEx._var_TotalSize(var _Data:TData; Index:word);
begin
  dtInteger(_Data, FTotalSize);
end;


procedure THIFileToolsEx._var_TransferedSize(var _Data:TData; Index:word);
begin
  dtInteger(_Data, FByteTransfered);
end;


end.
